#!/usr/local/bin/perl
# conn.cgi
# Open a PPTP connection and add its routes

require './pptp-client-lib.pl';
&foreign_require("net", "net-lib.pl");
&error_setup($text{'conn_err'});
&ReadParse();

# Get tunnel details
@tunnels = &list_tunnels();
($tunnel) = grep { $_->{'name'} eq $in{'tunnel'} } @tunnels;
$tunnel || &error($text{'conn_egone'});
&parse_comments($tunnel);
$tunnel->{'server'} || &error($text{'conn_einvalid'});

# Check if it is already active
@conns = &list_connected();
($conn) = grep { $_->[0] eq $in{'tunnel'} } @conns;
$conn && &error($text{'conn_ealready'});

$theme_no_table++;
$| = 1;
&header($text{'conn_title'}, "");
print "<hr>\n";

print "<p><b>",&text('conn_cmd', "<tt>$config{'pptp'} $tunnel->{'server'} ".
				 "call $in{'tunnel'}</tt>"),"</b><p>\n";

# Run the PPTP command, and wait for a new pppN interface to come up
%sifaces = map { $_->{'fullname'}, $_->{'address'} } &get_ppp_ifaces();
$start = time();
&system_logged("modprobe ip_gre >/dev/null 2>&1");
&system_logged("$config{'pptp'} ".quotemeta($tunnel->{'server'})." call ".
	       quotemeta($in{'tunnel'})." >/dev/null 2>&1 </dev/null &");
LOOP: while(time() - $start < $config{'timeout'}) {
	sleep(1);
	@nifaces = &get_ppp_ifaces();
	foreach $i (@nifaces) {
		if (!$sifaces{$i->{'fullname'}}) {
			$newiface = $i;
			last LOOP;
			}
		}
	}

# Find out if we were connected, or if it failed
if ($newiface) {
	# It worked! Tell the user, and add the routes
	print "<b>",&text('conn_ok', "<tt>$newiface->{'fullname'}</tt>",
			  "<tt>$newiface->{'address'}</tt>",
			  "<tt>$newiface->{'ptp'}</tt>"),"</b><p>\n";

	if (@{$tunnel->{'routes'}}) {
		@routes = &net::list_routes();
		($defroute) = grep { $_->{'dest'} eq '0.0.0.0' } @routes;
		$oldgw = $defroute->{'gateway'} if ($defroute);
		print "<b>$text{'conn_routes'}</b><pre>\n";
		foreach $r (@{$tunnel->{'routes'}}) {
			$cmd = "route $r";
			$cmd =~ s/TUNNEL_DEV/$newiface->{'fullname'}/g;
			$cmd =~ s/DEF_GW/$oldgw/g;
			$cmd =~ s/GW/$newiface->{'ptp'}/g;
			print $cmd,"\n";
			$out = &backquote_logged("$cmd 2>&1 </dev/null");
			print "<i>$out</i>";
			}
		print "</pre>\n";
		}
	}
else {
	# Must have timed out due to a failure
	print "<b>$text{'conn_timeout'}</b><p>\n";

	&foreign_require("syslog", "syslog-lib.pl");
	$sysconf = &syslog::get_config();
	foreach $c (@$sysconf) {
		next if ($c->{'tag'} || !$c->{'file'} || !-f $c->{'file'});
		local @st = stat($c->{'file'});
		if ($st[9] > $start) {
			# Was modified since start .. but by ppp or pptp?
			$tail = `tail -10 '$c->{'file'}'`;
			if ($tail =~ /ppp|pptp/) {
				$logs = $tail;
				last;
				}
			}
		}
	print "<pre>$logs</pre>\n";
	if ($logs =~ /mppe/) {
		print "<b>",&text('conn_mppe', "edit_opts.cgi"),"</b><p>\n";
		}
	}

# Save tunnel as default
&lock_file("$module_config_directory/config");
$config{'tunnel'} = $in{'tunnel'};
&write_file("$module_config_directory/config", \%config);
&unlock_file("$module_config_directory/config");

&webmin_log("conn", undef, $in{'tunnel'}, $newiface);
print "<hr>\n";
&footer("", $text{'index_return'});

sub get_ppp_ifaces
{
return grep { $_->{'fullname'} =~ /^ppp(\d+)$/ &&
	      $_->{'up'} && $_->{'address'} } &net::active_interfaces();
}

